"""
wagtail_editorjs
© 2024 Nigel van Keulen, goodadvice.it

A Wagtail integration for the Editor.js block editor.
Includes support for custom blocks, inline tools, and more.
Everything is dynamic and customizable.
"""

import distutils.version as pv



__version__ = 'v1.6.6'
VERSION = pv.LooseVersion(__version__)
