
"""
Hook used to register features with the editorjs widget.
"""
REGISTER_HOOK_NAME = "register_wagtail_editorjs_features"

"""
Hook used to build the configuration for the editorjs widget.
Any additional configuration or overrides can be done here.
"""
BUILD_CONFIG_HOOK = "editorjs_widget_build_config"

